import React, { Component } from 'react';
import './App.css';
import SearchForm from '../search-form/SearchForm.js';
import SearchResult from '../search-result/SearchResult.js';
import SearchHandler from '../search-handler/SearchHandler';
import Filter from '../filter/Filter.js';
import AppBar from '../app-bar/AppBar.js';

class App extends Component {
  constructor(props) {
    super(props);
    this.state = {
      searchFacets: {},
      searchResults: {
        docs: [],
        numFound: 0
      }
    };
    this.searchHandler = new SearchHandler({
      facetFields: [ 'group_name_s', 'author_ss', 'extra_Language', 'tags_ss', 'extra_PublicationTimestamp' ]
    });
    this.searchHandler.onResult(this.handleResult.bind(this));
  }

  componentDidMount() {
    this.searchHandler.search("*:*");
  }

  handleFilterChange = (filterData) => {
    this.searchHandler.applyFilters(filterData);
  }

  // on search form submit
  handleQuery = (data) => {
    this.searchHandler.search(data);
  }

  // on new results
  handleResult = (result) => {
    this.setState({searchResults: result, searchFacets: result.facets});
  }

  render() {
    return (
      <div className="App">
        <AppBar />
        <div className="container">
          <div className="row">
            <div className="col-xs-3"></div>
            <div className="col-xs-9">
              <SearchForm onQuery={this.handleQuery}/>
            </div>
          </div>
          <div className="row">
            <div className="col-xs-3 ilter">
              <Filter ref={filterComp => { this.filterComp = filterComp; }}
                facets={this.state.searchFacets}
                searchData={this.state.searchResults} onChange={this.handleFilterChange}/>
            </div>
            <div className = "col-xs-9 list">
              <SearchResult onResult={this.handleResult}
                searchData={this.state.searchResults}
                onPageChange={(page) => { this.searchHandler.changePage(page) }}
                ref={searchResultComp => { this.searchResultComp = searchResultComp; }} />
            </div>
          </div>
        </div>
      </div>

    );
  }
}

export default App;

// vi: ft=javascript.jsx
