import Axios from 'axios';
import qs from 'query-string';
import _ from 'lodash';

class SearchHandler {

  constructor(options) {
    this.queryStr = "";
    this.filters = {};
    this.start = 0;
    this.currentPage = 1;
    this.onResultHandler = () => {};

    options = options || {};
    this.rows = options.rows || 10;
    this.serverURL = options.serverURL || "http://localhost:8983";
    this.facetFields = options.facetFields || [];
    this.termFields = options.termFields || [];
  }

  changePage(page) {
    this.currentPage = page + 1;
    this.start = page * this.rows;
    this.search(this.queryStr, this.filters);
  }

  onResult(callback) {
    this.onResultHandler = callback;
  }

  applyFilters(filters) {
    this.search(this.queryStr, filters);
  }

  search(queryStr, filters) {
    this.queryStr = queryStr;
    this.filters = filters;
    filters = filters || {};

    let fq = Object.keys(filters)
                   .map((key) => `+${key}:${filters[key]}`)
                   .join(' ');

    let params = {
      q: queryStr,
      fq: fq,
      start: this.start,
      rows: this.rows,
      facet: true,
      'facet.field': this.facetFields
    };

    console.log('Querying Apache Solr', params);

    return Axios.get('http://localhost:8983/solr/b2find/select', {
      params: params,
      paramsSerializer: function(params) {
        return qs.stringify(params, { indices: false })
      }
    })
    .then((res) => {
      let docs = res.data.response.docs || [];
      let facets = {};
      this.facetFields.forEach((field) => {
        let facet = res.data.facet_counts.facet_fields[field];
        facets[field] = _.chunk(facet,2).map(fc => ({title: fc[0], count: fc[1]}));
      });
      
      let result = {
        docs: docs,
        clusters: res.data.clusters,
        numFound: res.data.response.numFound,
        facets: facets
      };
      console.log("Search Result: ", result);
      this.onResultHandler(result);
    })
    .catch((err) => {
      console.log("There was an error while querying Apache Solr server: ", err);
    });
  }

  fetchTerms(term, callback) {
    let params = {
      'terms.regex': term + '.*',
      'terms.regex.flag': 'case_insensitive',
      'terms.limit': 5,
      'terms.fl': this.termFields
    };
    return Axios.get(this.serverURL + '/solr/b2find/terms', {
      params: params,
      paramsSerializer: function(params) {
        return qs.stringify(params, { indices: false })
      }
    })
    .then((res) => {
      if (typeof callback === 'function') {
        let terms = {};
        this.termFields.forEach((field) => {
          let term = res.data.terms[field];
          terms[field] = _.chunk(term,2).map(fc => fc[0]);
        });
        callback(terms);
      }
    });
  }
}

export default SearchHandler;
